@@ -749,7 +749,7 @@ function run_k8s_kubelet {
         --root-dir=${KURYR_KUBERNETES_DATA_DIR}/kubelet"
 
     if [[ ${CONTAINER_ENGINE} == 'docker' ]]; then
-        command+=" --cgroup-driver $(docker info|awk '/Cgroup/ {print $NF}')"
+        command+=" --cgroup-driver $(docker info -f '{{.CgroupDriver}}')"
     elif [[ ${CONTAINER_ENGINE} == 'crio' ]]; then
         local crio_conf
         crio_conf=/etc/crio/crio.conf
@@ -847,7 +847,7 @@ spec:
     spec:
       containers:
       - name: coredns
-        image: coredns/coredns:1.5.0
+        image: quay.io/kuryr/coredns:1.5.0
         imagePullPolicy: Always
         args: [ "-conf", "/etc/coredns/Corefile" ]
         volumeMounts:
