@@ -13,15 +13,21 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+KURYR_FQDN = 'kuryr.openstack.org'
+
 K8S_API_BASE = '/api/v1'
 K8S_API_NAMESPACES = K8S_API_BASE + '/namespaces'
-K8S_API_CRD = '/apis/openstack.org/v1'
+K8S_API_CRD_VERSION = 'openstack.org/v1'
+K8S_API_CRD = '/apis/' + K8S_API_CRD_VERSION
 K8S_API_CRD_NAMESPACES = K8S_API_CRD + '/namespaces'
 K8S_API_CRD_KURYRNETS = K8S_API_CRD + '/kuryrnets'
 K8S_API_CRD_KURYRNETWORKS = K8S_API_CRD + '/kuryrnetworks'
 K8S_API_CRD_KURYRNETPOLICIES = K8S_API_CRD + '/kuryrnetpolicies'
+K8S_API_CRD_KURYRNETWORKPOLICIES = K8S_API_CRD + '/kuryrnetworkpolicies'
 K8S_API_CRD_KURYRLOADBALANCERS = K8S_API_CRD + '/kuryrloadbalancers'
+K8S_API_CRD_KURYRPORTS = K8S_API_CRD + '/kuryrports'
 K8S_API_POLICIES = '/apis/networking.k8s.io/v1/networkpolicies'
+K8S_API_NETWORKING = '/apis/networking.k8s.io/v1'
 
 K8S_API_NPWG_CRD = '/apis/k8s.cni.cncf.io/v1'
 
@@ -33,7 +39,9 @@ K8S_OBJ_POLICY = 'NetworkPolicy'
 K8S_OBJ_KURYRNET = 'KuryrNet'
 K8S_OBJ_KURYRNETWORK = 'KuryrNetwork'
 K8S_OBJ_KURYRNETPOLICY = 'KuryrNetPolicy'
+K8S_OBJ_KURYRNETWORKPOLICY = 'KuryrNetworkPolicy'
 K8S_OBJ_KURYRLOADBALANCER = 'KuryrLoadBalancer'
+K8S_OBJ_KURYRPORT = 'KuryrPort'
 
 K8S_POD_STATUS_PENDING = 'Pending'
 K8S_POD_STATUS_SUCCEEDED = 'Succeeded'
@@ -42,11 +50,13 @@ K8S_POD_STATUS_FAILED = 'Failed'
 K8S_ANNOTATION_PREFIX = 'openstack.org/kuryr'
 K8S_ANNOTATION_VIF = K8S_ANNOTATION_PREFIX + '-vif'
 K8S_ANNOTATION_LABEL = K8S_ANNOTATION_PREFIX + '-pod-label'
+K8S_ANNOTATION_IP = K8S_ANNOTATION_PREFIX + '-pod-ip'
 K8S_ANNOTATION_NAMESPACE_LABEL = K8S_ANNOTATION_PREFIX + '-namespace-label'
 K8S_ANNOTATION_LBAAS_SPEC = K8S_ANNOTATION_PREFIX + '-lbaas-spec'
 K8S_ANNOTATION_LBAAS_STATE = K8S_ANNOTATION_PREFIX + '-lbaas-state'
 K8S_ANNOTATION_NET_CRD = K8S_ANNOTATION_PREFIX + '-net-crd'
 K8S_ANNOTATION_NETPOLICY_CRD = K8S_ANNOTATION_PREFIX + '-netpolicy-crd'
+K8S_ANNOTATION_POLICY = K8S_ANNOTATION_PREFIX + '-counter'
 
 K8S_ANNOTATION_NPWG_PREFIX = 'k8s.v1.cni.cncf.io'
 K8S_ANNOTATION_NPWG_NETWORK = K8S_ANNOTATION_NPWG_PREFIX + '/networks'
@@ -59,7 +69,16 @@ K8S_ANNOTATION_OLD_DRIVER = 'old_driver'
 K8S_ANNOTATION_CURRENT_DRIVER = 'current_driver'
 K8S_ANNOTATION_NEUTRON_PORT = 'neutron_id'
 
+K8S_ANNOTATION_HEADLESS_SERVICE = 'service.kubernetes.io/headless'
+
+POD_FINALIZER = KURYR_FQDN + '/pod-finalizer'
 KURYRNETWORK_FINALIZER = 'kuryrnetwork.finalizers.kuryr.openstack.org'
+KURYRLB_FINALIZER = 'kuryr.openstack.org/kuryrloadbalancer-finalizers'
+SERVICE_FINALIZER = 'kuryr.openstack.org/service-finalizer'
+NETWORKPOLICY_FINALIZER = 'kuryr.openstack.org/networkpolicy-finalizer'
+
+KURYRPORT_FINALIZER = KURYR_FQDN + '/kuryrport-finalizer'
+KURYRPORT_LABEL = KURYR_FQDN + '/nodeName'
 
 K8S_OS_VIF_NOOP_PLUGIN = "noop"
 
