@@ -10,11 +10,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+import munch
+from unittest import mock
 
 from kuryr.lib import constants as kl_const
 from kuryr.lib import exceptions as kl_exc
-from neutronclient.common import exceptions as n_exc
+from openstack import exceptions as os_exc
 from oslo_config import cfg as oslo_cfg
 
 from kuryr_kubernetes import constants
@@ -31,7 +32,7 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
     def test_request_vif(self, m_to_vif):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
 
         pod = mock.sentinel.pod
         project_id = mock.sentinel.project_id
@@ -41,9 +42,12 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
         parent_port = mock.sentinel.parent_port
         trunk_id = mock.sentinel.trunk_id
         port_id = mock.sentinel.port_id
-        port = mock.MagicMock()
-        port.__getitem__.return_value = port_id
-        port_request = mock.sentinel.port_request
+        port = munch.Munch({'id': port_id})
+        port_request = {'project_id': project_id,
+                        'name': mock.sentinel.name,
+                        'network_id': mock.sentinel.network_id,
+                        'fixed_ips': mock.sentinel.fixed_ips,
+                        'admin_state_up': True}
         vlan_id = mock.sentinel.vlan_id
 
         vif = mock.Mock()
@@ -53,20 +57,18 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
         m_driver._get_trunk_id.return_value = trunk_id
         m_driver._get_port_request.return_value = port_request
         m_driver._add_subport.return_value = vlan_id
-        neutron.list_ports.return_value = {'ports': [parent_port]}
-        neutron.create_port.return_value = {'port': port}
+        os_net.ports.return_value = (p for p in [parent_port])
+        os_net.create_port.return_value = port
 
         self.assertEqual(vif, cls.request_vif(m_driver, pod, project_id,
                                               subnets, security_groups))
 
-        m_driver._get_parent_port.assert_called_once_with(neutron, pod)
+        m_driver._get_parent_port.assert_called_once_with(pod)
         m_driver._get_trunk_id.assert_called_once_with(parent_port)
         m_driver._get_port_request.assert_called_once_with(
             pod, project_id, subnets, security_groups)
-        neutron.create_port.assert_called_once_with(port_request)
-        m_driver._add_subport.assert_called_once_with(neutron,
-                                                      trunk_id,
-                                                      port_id)
+        os_net.create_port.assert_called_once_with(**port_request)
+        m_driver._add_subport.assert_called_once_with(trunk_id, port_id)
         m_to_vif.assert_called_once_with(port, subnets, vlan_id)
 
     @mock.patch(
@@ -74,7 +76,7 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
     def test_request_vifs(self, m_to_vif):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
 
         pod = mock.sentinel.pod
         project_id = mock.sentinel.project_id
@@ -91,7 +93,7 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
                          {'segmentation_id': 2,
                           'port_id': '',
                           'segmentation_type': 'vlan'}]
-        port = {'id': mock.sentinel.id}
+        port = munch.Munch({'id': mock.sentinel.id})
         vif = mock.sentinel.vif
 
         bulk_rq = {'ports': [port_request for _ in range(len(subports_info))]}
@@ -100,21 +102,21 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
         m_driver._get_trunk_id.return_value = trunk_id
         m_driver._create_subports_info.return_value = (port_request,
                                                        subports_info)
-        neutron.create_port.return_value = {'ports': [port, port]}
+        os_net.create_ports.return_value = (p for p in [port, port])
         m_to_vif.return_value = vif
 
         self.assertEqual([vif, vif], cls.request_vifs(
             m_driver, pod, project_id, subnets, security_groups, num_ports))
 
-        m_driver._get_parent_port.assert_called_once_with(neutron, pod)
+        m_driver._get_parent_port.assert_called_once_with(pod)
         m_driver._get_trunk_id.assert_called_once_with(parent_port)
         m_driver._create_subports_info.assert_called_once_with(
             pod, project_id, subnets, security_groups, trunk_id, num_ports,
             unbound=True)
-        neutron.create_port.assert_called_once_with(bulk_rq)
-        neutron.trunk_add_subports.assert_called_once_with(
-            trunk_id, {'sub_ports': subports_info})
-        neutron.delete_port.assert_not_called()
+        os_net.create_ports.assert_called_once_with(bulk_rq)
+        os_net.add_trunk_subports.assert_called_once_with(trunk_id,
+                                                          subports_info)
+        os_net.delete_port.assert_not_called()
 
         calls = [mock.call(port, subnets, info['segmentation_id'])
                  for info in subports_info]
@@ -123,7 +125,7 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
     def test_request_vifs_no_vlans(self):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        self.useFixture(k_fix.MockNetworkClient()).client
 
         pod = mock.sentinel.pod
         project_id = mock.sentinel.project_id
@@ -145,7 +147,7 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
                                               subnets, security_groups,
                                               num_ports))
 
-        m_driver._get_parent_port.assert_called_once_with(neutron, pod)
+        m_driver._get_parent_port.assert_called_once_with(pod)
         m_driver._get_trunk_id.assert_called_once_with(parent_port)
         m_driver._create_subports_info.assert_called_once_with(
             pod, project_id, subnets, security_groups,
@@ -154,7 +156,7 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
     def test_request_vifs_bulk_creation_exception(self):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
 
         pod = mock.sentinel.pod
         project_id = mock.sentinel.project_id
@@ -178,23 +180,23 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
         m_driver._get_trunk_id.return_value = trunk_id
         m_driver._create_subports_info.return_value = (port_request,
                                                        subports_info)
-        neutron.create_port.side_effect = n_exc.NeutronClientException
+        os_net.create_ports.side_effect = os_exc.SDKException
 
         self.assertRaises(
-            n_exc.NeutronClientException, cls.request_vifs,
+            os_exc.SDKException, cls.request_vifs,
             m_driver, pod, project_id, subnets, security_groups, num_ports)
 
-        m_driver._get_parent_port.assert_called_once_with(neutron, pod)
+        m_driver._get_parent_port.assert_called_once_with(pod)
         m_driver._get_trunk_id.assert_called_once_with(parent_port)
         m_driver._create_subports_info.assert_called_once_with(
             pod, project_id, subnets, security_groups,
             trunk_id, num_ports, unbound=True)
-        neutron.create_port.assert_called_once_with(bulk_rq)
+        os_net.create_ports.assert_called_once_with(bulk_rq)
 
     def test_request_vifs_trunk_subports_conflict(self):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
 
         pod = mock.sentinel.pod
         project_id = mock.sentinel.project_id
@@ -211,7 +213,7 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
                          {'segmentation_id': 2,
                           'port_id': '',
                           'segmentation_type': 'vlan'}]
-        port = {'id': mock.sentinel.id}
+        port = munch.Munch({'id': mock.sentinel.id})
 
         bulk_rq = {'ports': [port_request for _ in range(len(subports_info))]}
 
@@ -219,27 +221,26 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
         m_driver._get_trunk_id.return_value = trunk_id
         m_driver._create_subports_info.return_value = (port_request,
                                                        subports_info)
-        neutron.create_port.return_value = {'ports': [port, port]}
-        neutron.trunk_add_subports.side_effect = n_exc.Conflict
+        os_net.create_ports.return_value = (p for p in [port, port])
+        os_net.add_trunk_subports.side_effect = os_exc.ConflictException
 
-        self.assertRaises(n_exc.Conflict, cls.request_vifs,
-                          m_driver, pod, project_id, subnets,
-                          security_groups, num_ports)
+        self.assertEqual([], cls.request_vifs(m_driver, pod, project_id,
+                         subnets, security_groups, num_ports))
 
-        m_driver._get_parent_port.assert_called_once_with(neutron, pod)
+        m_driver._get_parent_port.assert_called_once_with(pod)
         m_driver._get_trunk_id.assert_called_once_with(parent_port)
         m_driver._create_subports_info.assert_called_once_with(
             pod, project_id, subnets, security_groups,
             trunk_id, num_ports, unbound=True)
-        neutron.create_port.assert_called_once_with(bulk_rq)
-        neutron.trunk_add_subports.assert_called_once_with(
-            trunk_id, {'sub_ports': subports_info})
-        neutron.delete_port.assert_called_with(port['id'])
+        os_net.create_ports.assert_called_once_with(bulk_rq)
+        os_net.add_trunk_subports.assert_called_once_with(trunk_id,
+                                                          subports_info)
+        os_net.delete_port.assert_called_with(port['id'])
 
     def test_request_vifs_trunk_subports_exception(self):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
 
         pod = mock.sentinel.pod
         project_id = mock.sentinel.project_id
@@ -256,7 +257,7 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
                          {'segmentation_id': 2,
                           'port_id': '',
                           'segmentation_type': 'vlan'}]
-        port = {'id': mock.sentinel.id}
+        port = munch.Munch({'id': mock.sentinel.id})
 
         bulk_rq = {'ports': [port_request for _ in range(len(subports_info))]}
 
@@ -264,27 +265,26 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
         m_driver._get_trunk_id.return_value = trunk_id
         m_driver._create_subports_info.return_value = (port_request,
                                                        subports_info)
-        neutron.create_port.return_value = {'ports': [port, port]}
-        neutron.trunk_add_subports.side_effect = n_exc.NeutronClientException
+        os_net.create_ports.return_value = (p for p in [port, port])
+        os_net.add_trunk_subports.side_effect = os_exc.SDKException
 
-        self.assertRaises(n_exc.NeutronClientException, cls.request_vifs,
-                          m_driver, pod, project_id, subnets,
-                          security_groups, num_ports)
+        self.assertEqual([], cls.request_vifs(m_driver, pod, project_id,
+                         subnets, security_groups, num_ports))
 
-        m_driver._get_parent_port.assert_called_once_with(neutron, pod)
+        m_driver._get_parent_port.assert_called_once_with(pod)
         m_driver._get_trunk_id.assert_called_once_with(parent_port)
         m_driver._create_subports_info.assert_called_once_with(
             pod, project_id, subnets, security_groups,
             trunk_id, num_ports, unbound=True)
-        neutron.create_port.assert_called_once_with(bulk_rq)
-        neutron.trunk_add_subports.assert_called_once_with(
-            trunk_id, {'sub_ports': subports_info})
-        neutron.delete_port.assert_not_called()
+        os_net.create_ports.assert_called_once_with(bulk_rq)
+        os_net.add_trunk_subports.assert_called_once_with(trunk_id,
+                                                          subports_info)
+        os_net.delete_port.assert_called_with(port['id'])
 
     def test_release_vif(self):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
         parent_port = mock.sentinel.parent_port
         trunk_id = mock.sentinel.trunk_id
 
@@ -295,16 +295,15 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
 
         cls.release_vif(m_driver, pod, vif)
 
-        m_driver._get_parent_port.assert_called_once_with(neutron, pod)
+        m_driver._get_parent_port.assert_called_once_with(pod)
         m_driver._get_trunk_id.assert_called_once_with(parent_port)
-        m_driver._remove_subport.assert_called_once_with(
-            neutron, trunk_id, vif.id)
-        neutron.delete_port.assert_called_once_with(vif.id)
+        m_driver._remove_subport.assert_called_once_with(trunk_id, vif.id)
+        os_net.delete_port.assert_called_once_with(vif.id)
 
     def test_release_vif_not_found(self):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
         parent_port = mock.sentinel.parent_port
         trunk_id = mock.sentinel.trunk_id
 
@@ -315,15 +314,13 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
         vif = mock.Mock()
         m_driver._port_vlan_mapping = {vif.id: vlan_id}
         self.assertTrue(vif.id in m_driver._port_vlan_mapping)
-        neutron.delete_port.side_effect = n_exc.PortNotFoundClient
 
         cls.release_vif(m_driver, pod, vif)
 
-        m_driver._get_parent_port.assert_called_once_with(neutron, pod)
+        m_driver._get_parent_port.assert_called_once_with(pod)
         m_driver._get_trunk_id.assert_called_once_with(parent_port)
-        m_driver._remove_subport.assert_called_once_with(
-            neutron, trunk_id, vif.id)
-        neutron.delete_port.assert_called_once_with(vif.id)
+        m_driver._remove_subport.assert_called_once_with(trunk_id, vif.id)
+        os_net.delete_port.assert_called_once_with(vif.id)
 
     def _test_get_port_request(self, m_to_fips, security_groups,
                                m_get_network_id, m_get_port_name,
@@ -346,18 +343,18 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
                                    True,
                                    group='kubernetes')
 
-        expected = {'port': {'project_id': project_id,
-                             'name': port_name,
-                             'network_id': network_id,
-                             'fixed_ips': fixed_ips,
-                             'device_owner': kl_const.DEVICE_OWNER,
-                             'admin_state_up': True}}
+        expected = {'project_id': project_id,
+                    'name': port_name,
+                    'network_id': network_id,
+                    'fixed_ips': fixed_ips,
+                    'device_owner': kl_const.DEVICE_OWNER,
+                    'admin_state_up': True}
 
         if security_groups:
-            expected['port']['security_groups'] = security_groups
+            expected['security_groups'] = security_groups
 
         if unbound:
-            expected['port']['name'] = constants.KURYR_PORT_NAME
+            expected['name'] = constants.KURYR_PORT_NAME
 
         ret = cls._get_port_request(m_driver, pod, project_id, subnets,
                                     security_groups, unbound)
@@ -417,7 +414,7 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
                          for i in range(num_ports)]
 
         m_driver._get_in_use_vlan_ids_set.return_value = in_use_vlan
-        m_driver._get_port_request.return_value = {'port': port}
+        m_driver._get_port_request.return_value = port
         m_allocate_seg_id.side_effect = [2, 3]
 
         port_res, subports_res = cls._create_subports_info(
@@ -450,7 +447,7 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
                           'segmentation_type': 'vlan'}]
 
         m_driver._get_in_use_vlan_ids_set.return_value = in_use_vlan
-        m_driver._get_port_request.return_value = {'port': port}
+        m_driver._get_port_request.return_value = port
         m_allocate_seg_id.side_effect = [
             2, kl_exc.SegmentationIdAllocationFailure
         ]
@@ -482,7 +479,7 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
         port = mock.sentinel.port
 
         m_driver._get_in_use_vlan_ids_set.return_value = in_use_vlan
-        m_driver._get_port_request.return_value = {'port': port}
+        m_driver._get_port_request.return_value = port
         m_allocate_seg_id.side_effect = kl_exc.SegmentationIdAllocationFailure
 
         port_res, subports_res = cls._create_subports_info(
@@ -516,7 +513,7 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
     def test_add_subport(self):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
         trunk_id = mock.sentinel.trunk_id
         subport = mock.sentinel.subport
         vlan_id = mock.sentinel.vlan_id
@@ -525,30 +522,29 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
                          'port_id': subport,
                          'segmentation_type': 'vlan'}]
         nested_vlan_vif.DEFAULT_MAX_RETRY_COUNT = 1
-        self.assertEqual(vlan_id, cls._add_subport(m_driver,
-                         neutron, trunk_id, subport))
+        self.assertEqual(vlan_id, cls._add_subport(m_driver, trunk_id,
+                                                   subport))
         m_driver._get_vlan_id.assert_called_once_with(trunk_id)
-        neutron.trunk_add_subports.assert_called_once_with(
-            trunk_id, {'sub_ports': subport_dict})
+        os_net.add_trunk_subports.assert_called_once_with(trunk_id,
+                                                          subport_dict)
 
     def test_add_subport_get_vlanid_failure(self):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        self.useFixture(k_fix.MockNetworkClient()).client
         trunk_id = mock.sentinel.trunk_id
         subport = mock.sentinel.subport
-        m_driver._get_vlan_id.side_effect = n_exc.NeutronClientException
+        m_driver._get_vlan_id.side_effect = os_exc.SDKException
         nested_vlan_vif.DEFAULT_MAX_RETRY_COUNT = 1
-        self.assertRaises(
-            n_exc.NeutronClientException, cls._add_subport,
-            m_driver, neutron, trunk_id, subport)
+        self.assertRaises(os_exc.SDKException, cls._add_subport, m_driver,
+                          trunk_id, subport)
 
         m_driver._get_vlan_id.assert_called_once_with(trunk_id)
 
     def test_add_subport_with_vlan_id_conflict(self):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
         trunk_id = mock.sentinel.trunk_id
         subport = mock.sentinel.subport
         vlan_id = mock.sentinel.vlan_id
@@ -556,25 +552,37 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
         subport_dict = [{'segmentation_id': vlan_id,
                          'port_id': subport,
                          'segmentation_type': 'vlan'}]
-        neutron.trunk_add_subports.side_effect = n_exc.Conflict
+        os_net.add_trunk_subports.side_effect = os_exc.ConflictException
         nested_vlan_vif.DEFAULT_MAX_RETRY_COUNT = 1
-        self.assertRaises(n_exc.Conflict, cls._add_subport, m_driver,
-                          neutron, trunk_id, subport)
+        self.assertRaises(os_exc.ConflictException, cls._add_subport, m_driver,
+                          trunk_id, subport)
 
-        neutron.trunk_add_subports.assert_called_once_with(
-            trunk_id, {'sub_ports': subport_dict})
+        os_net.add_trunk_subports.assert_called_once_with(trunk_id,
+                                                          subport_dict)
 
     def test__remove_subports(self):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
         trunk_id = mock.sentinel.trunk_id
         subport_id = mock.sentinel.subport_id
         subportid_dict = [{'port_id': subport_id}]
-        cls._remove_subports(m_driver, neutron, trunk_id, [subport_id])
+        cls._remove_subports(m_driver, trunk_id, [subport_id])
 
-        neutron.trunk_remove_subports.assert_called_once_with(
-            trunk_id, {'sub_ports': subportid_dict})
+        os_net.delete_trunk_subports.assert_called_once_with(trunk_id,
+                                                             subportid_dict)
+
+    def test__remove_subports_duplicate(self):
+        cls = nested_vlan_vif.NestedVlanPodVIFDriver
+        m_driver = mock.Mock(spec=cls)
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
+        trunk_id = mock.sentinel.trunk_id
+        subport_id = mock.sentinel.subport_id
+        subportid_dict = [{'port_id': subport_id}]
+        cls._remove_subports(m_driver, trunk_id, [subport_id, subport_id])
+
+        os_net.delete_trunk_subports.assert_called_once_with(trunk_id,
+                                                             subportid_dict)
 
     @mock.patch('kuryr.lib.segmentation_type_drivers.allocate_segmentation_id')
     def test_get_vlan_id(self, mock_alloc_seg_id):
@@ -612,18 +620,14 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
     def test_get_in_use_vlan_ids_set(self):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
         m_driver = mock.Mock(spec=cls)
-        neutron = self.useFixture(k_fix.MockNeutronClient()).client
+        os_net = self.useFixture(k_fix.MockNetworkClient()).client
 
         vlan_ids = set()
         trunk_id = mock.sentinel.trunk_id
         vlan_ids.add('100')
 
-        port = mock.MagicMock()
-        port.__getitem__.return_value = '100'
-        trunk_obj = mock.MagicMock()
-        trunk_obj.__getitem__.return_value = [port]
-        trunk = mock.MagicMock()
-        trunk.__getitem__.return_value = trunk_obj
-        neutron.show_trunk.return_value = trunk
+        port = munch.Munch({'segmentation_id': '100'})
+        trunk_obj = munch.Munch({'sub_ports': [port]})
+        os_net.get_trunk.return_value = trunk_obj
         self.assertEqual(vlan_ids,
                          cls._get_in_use_vlan_ids_set(m_driver, trunk_id))
