@@ -42,7 +42,10 @@ VALID_MULTI_POD_POOLS_OPTS = {'noop': ['neutron-vif',
                               'nested': ['nested-vlan'],
                               }
 DEFAULT_TIMEOUT = 500
-DEFAULT_INTERVAL = 3
+DEFAULT_INTERVAL = 1
+DEFAULT_JITTER = 3
+MAX_BACKOFF = 60
+MAX_ATTEMPTS = 10
 
 subnet_caching_opts = [
     cfg.BoolOpt('caching', default=True),
@@ -94,33 +97,33 @@ def convert_netns(netns):
         return netns
 
 
-def get_pod_unique_name(pod):
-    """Returns a unique name for the pod.
+def get_res_unique_name(resource):
+    """Returns a unique name for the resource like pod or CRD.
 
-    It returns a pod unique name for the pod composed of its name and the
-    namespace it is running on.
+    It returns a unique name for the resource composed of its name and the
+    namespace it is created in or just name for cluster-scoped resources.
 
-    :returns: String with namespace/name of the pod
+    :returns: String with <namespace/>name of the resource
     """
-    return "%(namespace)s/%(name)s" % pod['metadata']
+    try:
+        return "%(namespace)s/%(name)s" % resource['metadata']
+    except KeyError:
+        return "%(name)s" % resource['metadata']
 
 
 def check_suitable_multi_pool_driver_opt(pool_driver, pod_driver):
     return pod_driver in VALID_MULTI_POD_POOLS_OPTS.get(pool_driver, [])
 
 
-def exponential_sleep(deadline, attempt, interval=DEFAULT_INTERVAL):
+def exponential_sleep(deadline, attempt, interval=DEFAULT_INTERVAL,
+                      max_backoff=MAX_BACKOFF, jitter=DEFAULT_JITTER):
     """Sleep for exponential duration.
 
-    This implements a variation of exponential backoff algorithm [1] and
-    ensures that there is a minimal time `interval` to sleep.
-    (expected backoff E(c) = interval * 2 ** c / 2).
-
-    [1] https://en.wikipedia.org/wiki/Exponential_backoff
-
     :param deadline: sleep timeout duration in seconds.
     :param attempt: attempt count of sleep function.
     :param interval: minimal time interval to sleep
+    :param max_backoff: maximum time to sleep
+    :param jitter: max value of jitter added to the sleep time
     :return: the actual time that we've slept
     """
     now = time.time()
@@ -129,7 +132,8 @@ def exponential_sleep(deadline, attempt, interval=DEFAULT_INTERVAL):
     if seconds_left <= 0:
         return 0
 
-    to_sleep = random.randint(1, 2 ** attempt - 1) * interval
+    to_sleep = exponential_backoff(attempt, interval, max_backoff=max_backoff,
+                                   jitter=jitter)
 
     if to_sleep > seconds_left:
         to_sleep = seconds_left
@@ -141,6 +145,31 @@ def exponential_sleep(deadline, attempt, interval=DEFAULT_INTERVAL):
     return to_sleep
 
 
+def exponential_backoff(attempt, interval=DEFAULT_INTERVAL,
+                        max_backoff=MAX_BACKOFF, jitter=DEFAULT_JITTER):
+    """Return exponential backoff duration with jitter.
+
+    This implements a variation of exponential backoff algorithm [1] (expected
+    backoff E(c) = interval * 2 ** attempt / 2).
+
+    [1] https://en.wikipedia.org/wiki/Exponential_backoff
+    """
+
+    if attempt >= MAX_ATTEMPTS:
+        # No need to calculate very long intervals
+        attempt = MAX_ATTEMPTS
+
+    backoff = 2 ** attempt * interval
+
+    if max_backoff is not None and backoff > max_backoff:
+        backoff = max_backoff
+
+    if jitter:
+        backoff += random.randint(0, jitter)
+
+    return backoff
+
+
 def get_node_name():
     # leader-elector container based on K8s way of doing leader election is
     # assuming that hostname it sees is the node id. Containers within a pod
@@ -233,8 +262,12 @@ def has_limit(quota):
 
 def is_available(resource, resource_quota):
     availability = resource_quota['limit'] - resource_quota['used']
-    if availability <= 0:
-        LOG.error("Quota exceeded for resource: %s", resource)
+    if availability <= 3:
+        LOG.warning("Neutron quota low for %s. Used %d out of %d limit.",
+                    resource, resource_quota['limit'], resource_quota['used'])
+    elif availability <= 0:
+        LOG.error("Neutron quota exceeded for %s. Used %d out of %d limit.",
+                  resource, resource_quota['limit'], resource_quota['used'])
         return False
     return True
 
@@ -243,9 +276,11 @@ def has_kuryr_crd(crd_url):
     k8s = clients.get_kubernetes_client()
     try:
         k8s.get(crd_url, json=False, headers={'Connection': 'close'})
+    except exceptions.K8sResourceNotFound:
+        LOG.error('CRD %s does not exists.', crd_url)
     except exceptions.K8sClientException:
-        LOG.exception("Kubernetes Client Exception fetching"
-                      " CRD. %s" % exceptions.K8sClientException)
+        LOG.exception('Error fetching CRD %s, assuming it does not exist.',
+                      crd_url)
         return False
     return True
 
@@ -338,24 +373,39 @@ def get_endpoints_link(service):
     return "/".join(link_parts)
 
 
-def has_port_changes(service, lbaas_spec):
+def get_service_link(endpoints):
+    endpoints_link = endpoints['metadata']['selfLink']
+    link_parts = endpoints_link.split('/')
+
+    if link_parts[-2] != 'endpoints':
+        raise exceptions.IntegrityError(
+            f"Unsupported endpoints link: {endpoints_link}")
+    link_parts[-2] = 'services'
+
+    return "/".join(link_parts)
+
+
+def has_port_changes(service, loadbalancer_crd):
+    if not loadbalancer_crd:
+        return False
     link = service['metadata']['selfLink']
+    svc_port_set = service['spec'].get('ports')
 
-    fields = obj_lbaas.LBaaSPortSpec.fields
-    svc_port_set = {tuple(port[attr] for attr in fields)
-                    for port in get_service_ports(service)}
-
-    spec_port_set = {tuple(getattr(port, attr)
-                     for attr in fields
-                     if port.obj_attr_is_set(attr))
-                     for port in lbaas_spec.ports}
-
-    if svc_port_set != spec_port_set:
-        LOG.debug("LBaaS spec ports %(spec_ports)s != %(svc_ports)s "
-                  "for %(link)s" % {'spec_ports': spec_port_set,
-                                    'svc_ports': svc_port_set,
-                                    'link': link})
-    return svc_port_set != spec_port_set
+    for port in svc_port_set:
+        port['targetPort'] = str(port['targetPort'])
+    spec_port_set = loadbalancer_crd['spec'].get('ports', [])
+    if spec_port_set:
+        if len(svc_port_set) != len(spec_port_set):
+            return True
+        pairs = zip(svc_port_set, spec_port_set)
+        diff = any(x != y for x, y in pairs)
+        if diff:
+            LOG.debug("LBaaS spec ports %(spec_ports)s != %(svc_ports)s "
+                      "for %(link)s" % {'spec_ports': spec_port_set,
+                                        'svc_ports': svc_port_set,
+                                        'link': link})
+        return diff
+    return False
 
 
 def get_service_ports(service):
@@ -376,3 +426,18 @@ def get_service_subnet_version():
         LOG.exception("Service subnet %s not found", svc_subnet_id)
         raise
     return svc_subnet.ip_version
+
+
+def clean_lb_crd_status(loadbalancer_name):
+    namespace, name = loadbalancer_name.split('/')
+    k8s = clients.get_kubernetes_client()
+    try:
+        k8s.patch_crd('status', f'{constants.K8S_API_CRD_NAMESPACES}'
+                      f'/{namespace}/kuryrloadbalancers/{name}', {})
+    except exceptions.K8sResourceNotFound:
+        LOG.debug('KuryrLoadbalancer CRD not found %s',
+                  name)
+    except exceptions.K8sClientException:
+        LOG.exception('Error updating KuryrLoadbalancer CRD %s',
+                      name)
+        raise
